''
''******************************
''*                            *
''*   Caliper scales counter   *
''*                            *
''*   Richard Schultz          *
''*                            *
''*   11 August 2008           *
''*                            *
''******************************
''

'' This program reads only the 'relative value' of the caliper scale,
'' discarding the 'absolute value'. It outputs signed integers in
'' binary format.


VAR

  long                Pos             ' holds address of hub memory buffer
  byte                Cog             ' cog (ID+1) that is running Update 


PUB Start(PosAddr): Pass

'' PosAddr  = start address of a buffer of longs where each counter's position is to be stored.
''
'' RETURNS:    True if successful, False otherwise.
''
  Pos := PosAddr                      ' Pos with be passed into COG as PAR
                                      ' it will hold starting address of
                                      ' the Hub memory buffer
  stop                                ' jump to 'stop'  
  pass := (Cog := cognew(@Update, Pos) + 1) > 0
  

PUB Stop

'' Stop the encoder-reading cog, if there is one.
''

  if Cog > 0
    cogstop(Cog~ - 1)


'=============================================== assembly code ====================================================

DAT

                        org     0

Update                        

' set i/o for input
' -----------------
' give start of hub memory buffer to MPosAddr in the cog
' ------------------------------------------------------
' set clock and data masks
' ------------------------

initialize         
                        mov     dira,           #0                              ' all I/O pins are inputs
    init_scale_1                                                                ' 1st counter initialized
                        mov     num_scales,     #3                              ' number of scales to read
                        mov     MPosAddr,       PAR                             ' save hub mem. addr. into MPosAddr
                        mov     clock_mask,     clk_mask                        ' set initial clock_mask (pin 0)
                        mov     data_mask,      dat_mask                        ' set initial data_mask (pin 1)
                        jmp     #sync_data                                      ' jump over 'shifting mask bits left' for
                                                                                ' initial setup
    init_scale_1_2                                                              ' 2nd + 3rd counters initialized
                        add     MPosAddr,       #4                              ' add 4 to MPosAddr                          
                        shl     clock_mask,     #2                              ' shift clock_mask left by two bits
                        shl     data_mask,      #2                              ' shift data_mask left by two bits
                                                                                ' the masks determine which i/o pin will be sampled
                                                                                
                                                                                
' look for contiguous period of time when clock_bit = 1 for ~100us
' ------------------------------------------------------------------
' this just preceeds 'relative caliper data'
' ------------------------------------------

sync_data                                                                       ' look for clock = 1
                        mov     clock_bit,      ina                             ' input clock
                        and     clock_bit,      clock_mask                      ' mask all bits except clock bit
                        tjz     clock_bit,      #sync_data                      ' return to sync_data if clock = 0
                                                                                ' start data bit collection when clock_bit = 1
                        mov     index,          sync_delay                      ' load index with ~100us sync_delay
    wait                    
                        mov     clock_bit,      ina                             ' input clock (should be = 1), loop for ~100us
                        and     clock_bit,      clock_mask                      ' mask all bits except clock_bit
                        tjz     clock_bit,      #sync_data                      ' if clock = 0 return to sync_data and start over
                        djnz    index,          #wait                           ' if index > 0, loop to :wait
                                                                                ' once ~100us delay completed, load variables
                        mov     scale_data,     zero                            ' zero scale_data buffer when ~100us delay is completed
                        mov     index,          #24                             ' set index = 24 bits of data to capture

' look for valid_data_clock = 0 
' -----------------------------

valid_data_clock                        
                        mov     clock_bit,      ina                             ' input clock
                        and     clock_bit,      clock_mask                      ' mask all bits except clock bit
                        tjnz    clock_bit,      #valid_data_clock               ' when clock = 0, data is valid
                                                                                ' initialize variables prior to getting data                        
                        shl     scale_data,     #1                              ' shift scale_data left 1 bit
                        mov     bit_cnt,        zero                            ' zero out bit_count
                        mov     counter,        #3                              ' set up counter to loop 3 times                        

' move valid data into data_bit, majority average, and shift into scale_data 
' --------------------------------------------------------------------------

     get_data                                                                   ' get data_bit, sample 3 times
                                                                                ' data_bit is 'positive' when = 0
                        mov     data_bit,       ina                             ' input data
                        and     data_bit,       data_mask       wz              ' mask all bits except for data bit 
              if_z      add     bit_cnt,        #1                              ' if wz = 0 because data_bit = 0
                                                                                ' add 1 to result
                        djnz    counter,        #get_data                       ' loop x 3
                                                                                ' (do a majority average on bit_cnt for 
                                                                                ' the first 2 bits, if 00 or 01 add 0, if 
                                                                                ' 10 or 11 add 1 ... to scale_data)
                        and     bit_cnt,        bit_cnt_mask    wz              ' if 2nd bit = 1 add 1 to scale_data
              if_nz     add     scale_data,     #1                              ' if 2nd bit = 0 add nothing to scale_data
                                                                                
' wait for clock_bit to become valid, then jump to :valid_data_clock
' ------------------------------------------------------------------

     next_clock
                        mov     clock_bit,      ina                             ' input clock
                        and     clock_bit,      clock_mask                      ' mask all bits except clock bit
                        tjz     clock_bit,      #next_clock                     ' wait for clock_bit = 1, then
                                                                                ' go get next data bit 
                        djnz    index,          #valid_data_clock               ' continue collecting 24 bits of data
                                                                                ' when done, prepare data for transfer

' once all of the data bits are collected into scale_data as 'lsb --> msb', 
' -------------------------------------------------------------------------
' reverse scale_data 'msb --> lsb' and test for negative number. If number  
' ------------------------------------------------------------------------
' is negative, decode 2's complement, preserve negative sign, shift data 
' ----------------------------------------------------------------------
' right by 8 bits. If number is positive shift data right by 8 bits.  Write
' -------------------------------------------------------------------------
' scale_data to hub memory location, MPosAddress.
' -----------------------------------------------
                                                                                
                        rev     scale_data,     #0                              ' reverse the 32 bits
                        mov     temp_scale,     scale_data                      ' make a copy of scale_data for use if # is negative 
                        test    scale_data,     sign_bit        wz              ' check msb (0-31) bit, 1 or 0?                        
                        shr     scale_data,     #8                              ' shift scale data right 8 bits
              if_nz     call    #neg_num                                        ' if msb = 1 then scale_data is neg.
                                                                                ' number, go to neg_num subroutine              
                        wrlong  scale_data,     MPosAddr                        ' write scale_data to hub memory
                        djnz    num_scales,     #init_scale_1_2                 ' return to read scales 2 + 3
end                     jmp     #init_scale_1                                   ' return to init_scale_1 to restart reading
                                                                                ' all of the scales 


' SUBROUTINES
' ===========

' neg_num subroutine...unravels the 2's complement negative number
' ----------------------------------------------------------------
' into a signed integer value in binary format
' --------------------------------------------                                                  
                                                                                
neg_num                                                                         ' negative number is in temp_scale
                        mov     scale_data,     max_num                         ' load scale_data with $FF_FF_FF_FF
                        andn    scale_data,     temp_scale                      ' get NOT of temp_scale AND with scale_data
                        shr     scale_data,     #8                              ' shift scale_data right 8 bits
                        add     scale_data,     #1                              ' add 1 to scale_data
                        neg     scale_data,     scale_data                      ' add negative sign back (msb = 1) 
neg_num_ret             ret                                                     ' return and write scale_data to hub memory


' These variables are in cog ram!
' ===============================
                        
' Initialized data 
' ----------------

clk_mask      long    %00000000_00000000_00000000_00000001
dat_mask      long    %00000000_00000000_00000000_00000010
bit_cnt_mask  long    %00000000_00000000_00000000_00000010 
sync_delay    long    $00_00_01_2C
max_num       long    $FF_FF_FF_FF_
sign_bit      long    $80_00_00_00
zero          long    $00_00_00_00

' Uninitialized data
' ------------------

index         res     1         ' address of index
MPosAddr      res     1         ' address of current encoder position counter (Main Memory)
clock_bit     res     1         ' clock_bit address
data_bit      res     1         ' data_bit address
scale_data    res     1         ' address of data buffer       
temp_scale    res     1         ' temporary scale data
counter       res     1         ' loop x 3
bit_cnt       res     1         ' sum of sampling bit data x 3
clock_mask    res     1         ' mask for clock bit
data_mask     res     1         ' mask for data bit
num_scales    res     1         ' number of scales to read

                        fit
                          